#define COMMAND_MAX 30000

enum { 
	Load, 
	LoadMap,
	LoadMapFile,
	ReSetEngine, 
	Show,          // Show the current object
	Hide,          // Hide the current object
	Clear,         // clear out entire space
	InfoDimension, 
	Link,  // first pass on new feature, links two visual objects

	// Object Creation
	StartData,  // startdata <name> 
	ModelSetCommand,   // modelset <name> <N>
	ModelSwitch,
	EndData, 
	Data, 

	// Motion Controls
	Left, 
	Right, 
	Up, 
	Down, 
	In, 
	Out, 
	RotateX,
	RotateXcc,
	RotateY,
	RotateYcc,
	RotateZ,
	RotateZcc,
	MarkX,
	MarkY,
	MarkZ,
	SpaceX,
	SpaceY,
	SpaceZ,
	SetMarker,

	// Scale Controls
	Taller, 
	Shorter, 
	Bigger, 
	Smaller, 
	Fatter, 
	Thinner, 
	Scale, 

	SetGroup,
	ClearGroup,
	SetMax, 
	Cycle, 
	FadeOut, 
	FadeIn, 
	Inline, 
	Debug, 
	Label,
	StartTemplate, 
	EndTemplate, 
	TemplateData, 
	StartFunction,
	EndFunction,
	CallFunction,
	FunctionData,
	SetInfo,
	TestDEF,        // Test if a particular node DEF exists (results logged)

	// Generic object attribute controls
	FastMode,
	SlowMode,
	SetObject, 
	SetVizProperty,
	SetVizValue
 };

class Command {
public:
	Command( int command, char* p1, char* p2 );
	~Command();
	int getValue() { return( command ); }
	char* getParam() { return( param1 ); }
	char* getParam2() { return( param2 ); }
	char* takeParam() { char* result = param1; param1 = (char*)0; return result; }
private:
	int command;
	char* param1;
	char* param2;
};

class InternalCommandQueue {
public:
	InternalCommandQueue();
	void queue( char* command );
	Command* getNextCommand();
	Command* unconditionalGetNextCommand();
	void load( const char* filename );
	void debugOn() { debug = true; }
	void addCommand( int command, const char* p );
	void suspend() { suspended = true; }
	void resume() { suspended = false; }
	bool hasCommand();
	char* toStr( int command );
protected:
	bool suspended;
	bool debug;
	bool inData;
	bool inTemplate;
	bool inFunction;
	int incValue( int v );
	bool spaceAvailable();
	int currentIdx;  // next place to put a command
	int nextCommandIdx; // next command to execute
	int command[COMMAND_MAX];
	char* param1[COMMAND_MAX];
	char* param2[COMMAND_MAX];
};

class CommandQueue : public InternalCommandQueue {
public:
	CommandQueue();
	enum { Insert, Append };
	void insertMode() { mode = Insert; suspended = true; }  // mode controls how commands get put in queue
	void appendMode() { mode = Append; suspended = false; }  
	Command* getNextCommand();
	Command* unconditionalGetNextCommand();
	void addCommand( int command, const char* p );
private:
	int mode;
	InternalCommandQueue insertQueue;
};
