/*
   MindtelSocket.cpp

  Minimal socket interface for secure socket data transfer.

  */

#include <stdio.h>
#include <io.h>
#include <winsock2.h>
#include "MindtelSocket.h"
#include "log.h"

// windows specific garbage
static bool initialized = false;

static int init() {
	if ( !initialized ) {
		WORD wVersionRequested;
		WSADATA wsaData;
		int err;
 
		wVersionRequested = MAKEWORD( 2, 2 );
 
		err = WSAStartup( wVersionRequested, &wsaData );
		if ( err != 0 ) {
			/* Tell the user that we could not find a usable */
			/* WinSock DLL.                                  */
//			printf("WSAStartup didn't fucking work!\n");
			return(-1);
		}
 
		/* Confirm that the WinSock DLL supports 2.2.*/
		/* Note that if the DLL supports versions greater    */
		/* than 2.2 in addition to 2.2, it will still return */
		/* 2.2 in wVersion since that is the version we      */
		/* requested.                                        */
		if ( LOBYTE( wsaData.wVersion ) != 2 ||
			HIBYTE( wsaData.wVersion ) != 2 ) {
			/* Tell the user that we could not find a usable */
			/* WinSock DLL.                                  */
			WSACleanup( );
			return(-1); 
		}
 
	}
	initialized = true;
	return( 0 );
}

static void bzero( char* ptr, int size ) {
	for ( int i = 0; i < size; i++ ) {
		*ptr++ = 0;
	}
}


/*
 *  MindtelSocket::MindtelSocket
 *
 *  Purpose:
 *    Initialize the networking (windows only), then set up the socket.
 */

MindtelSocket::MindtelSocket() {
	if (init() != 0) {
		printf( "Sockets are fucked up on this machine.  Thanks, Microsoft!\n" );
		exit( 0 );
	}

	/* The WinSock DLL is acceptable. Proceed. */
}


/*
 *  MindtelSocket::Send
 * 
 *  Purpose:
 *    Send data out connected socket
 *
 *  Input parameters:
 *    buf -- pointer to memory to send out socket
 *    buflen -- how many bytes to send
 */

void MindtelSocket::Send( const char* buf, int buflen ) {
	send( sockfd, buf, buflen, 0 );
}


/*
 *  MindtelSocket::Close
 *
 *  Purpose:
 *    Close the socket connection
 */

void MindtelSocket::Close() {
	close( sockfd );
}


/*
 *  MindtelSocket::Recv
 * 
 *  Purpose:
 *    Recv data from connected socket
 *
 *  Input parameters:
 *    buf -- pointer to receive destination memory
 *    buflen -- how many bytes to receive
 */

int MindtelSocket::Recv( char* buf, int buflen ) {
	log_messagei2( "Recv socket: ", sockfd, buflen );
	int result = recv( sockfd, buf, buflen, 0 );
	if ( result >= 0 ) {
		*(buf + result) = 0;  // for those callers that are string based
	}
	return( result );
}


/*
 *  MindtelSocketClient::MindtelSocketClient
 *
 *  Purpose:
 *    Constructor for side initiating a socket connection.
 *
 *  Input Parameters:
 *    ip -- ip address of the remote machine
 *    portNumber -- port to connect to
 */

MindtelSocketClient::MindtelSocketClient( char* ipin, int portNo)
: MindtelSocket() {
	memset( ip, 0, 30 );
	strncpy( ip, ipin, 29 );
	portNumber = portNo;
}


/*
 *  MindtelSocketClient::Connect( ip, portNumber ) 
 *
 *  Purpose:
 *    Set up TCP socket connection to a remote machine.
 *
 *  Return Value:
 *    true if the connection succeeds, otherwise false
 */

bool MindtelSocketClient::Connect() {
	sockfd = socket(AF_INET, SOCK_STREAM, 0);

	struct sockaddr_in servaddr;
	bzero((char*)&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(portNumber);
	servaddr.sin_addr.S_un.S_addr = inet_addr(ip);

    if (connect(sockfd, (const struct sockaddr*) &servaddr, sizeof(servaddr)) == -1) {
		return( false );
	} else {
		return( true );
	}
}


void MindtelSocketClient::GET( char* page ) {
	char buf[500];
	sprintf( buf, "GET %s HTTP/1.0\r\n\r\n", page );
	Send( buf, strlen( buf ));
}

int grabresult( char* s ) {
	while ( *s ) {
		if ( strncmp( s, "<int>", 5 ) == 0 ) {
			return( atoi( s + 5 ));
		}
		s++;
	}
	return( -1 );
}

int MindtelSocketClient::simpleXmlRpc( char* page, char* method, char* params ) {
	static char* bodyFormat = "<?xml version='1.0'?>\n<methodCall>\n<methodName>%s</methodName>\n<params>\n<param>\n<value><string>%s</string></value>\n</param>\n</params>\n</methodCall>\n";
	static char* headerFormat = "POST %s HTTP/1.0\r\nHost: nomatter\r\nUser-Agent: xmlrpclib.py/0.9.8 (by www.pythonware.com)\r\nContent-Type: text/xml\r\nContent-Length: %d\r\n\r\n";
	char sendStuff[1000];
	sprintf( sendStuff, bodyFormat, method, params );
	int len = strlen( sendStuff );
	char headerStuff[300];
	sprintf( headerStuff, headerFormat, page, len );
	Send( headerStuff, strlen( headerStuff ));
	Send( sendStuff, strlen( sendStuff ));
				char result[1000];
				while ( Recv( &result[0], 999 ) > 0 ) {
					int x = grabresult( &result[0] );
					if ( x != -1 ) {
						return( x );
					}
					//printf( result );
				}
				//printf( "\nThat was it!\n" );
	return( 0 );
}

int MindtelSocketClient::tidesXmlRpc( char* page, char* method, char* params ) {
	static char* bodyFormat = "<?xml version='1.0'?>\n<methodCall>\n<methodName>%s</methodName>\n<params>\n<param>\n<value><struct>%s</struct></value>\n</param>\n</params>\n</methodCall>\n";
	static char* headerFormat = "POST %s HTTP/1.0\r\nHost: nomatter\r\nUser-Agent: xmlrpclib.py/0.9.8 (by www.pythonware.com)\r\nContent-Type: text/xml\r\nContent-Length: %d\r\n\r\n";
	char sendStuff[2000];
	sprintf( sendStuff, bodyFormat, method, params );
	int len = strlen( sendStuff );
	char headerStuff[300];
	sprintf( headerStuff, headerFormat, page, len );
	Send( headerStuff, strlen( headerStuff ));
	Send( sendStuff, strlen( sendStuff ));
				char result[1000];
				while ( Recv( &result[0], 999 ) > 0 ) {
					int x = grabresult( &result[0] );
					if ( x != -1 ) {
						return( x );
					}
					//printf( result );
				}
				//printf( "\nThat was it!\n" );
	return( 0 );
}


MindtelSocketServer::MindtelSocketServer() : MindtelSocket() {
	internalValue = 0;
}

/**
 *  MindtelSocketServer::MindtelSocketServer
 *
 *  Constructor for a simple server class.
 */
MindtelSocketServer::MindtelSocketServer( int port ) : MindtelSocket() {
	running = false;

	/* The WinSock DLL is acceptable. Proceed. */
    listenfd = socket(AF_INET, SOCK_STREAM, 0);

    bzero((char*)&servaddr, sizeof(servaddr));
    servaddr.sin_family      = AF_INET;
    servaddr.sin_addr.s_addr = htonl( INADDR_ANY );
    servaddr.sin_port        = htons( port );

	//log_message( "calling bind");

    bind(listenfd, (const struct sockaddr*)&servaddr, sizeof(servaddr));

	//log_message( "calling listen" );
	listen( listenfd, 5 );
}

bool MindtelSocketServer::AcceptConnection() {
	int clilen = 0;
	bzero( (char*)&cliaddr, sizeof( cliaddr ));
    clilen = sizeof(cliaddr);
	printf( "accept connection, listenfd is %d\n", listenfd );
	if ( (sockfd = accept(listenfd, (struct sockaddr*)&cliaddr, &clilen)) < 0) {
		log_error( "accept error" );
		return( false );
	}
	return( true );
}

#include <process.h>
extern bool doPost( char* pageToGet, char* buf, MindtelSocketServer* theServer );

void MyThreadProc( void* x ) {
	printf( "*********** THREAD START ***********\n");
	// make internal to thread copy
	MindtelSocketServer* threadInternal = (MindtelSocketServer*)x;

	threadInternal->StartUp();
	int recvResult2 = 1;
	while ( threadInternal->IsRunning() ) {
			char buf[5000];
			buf[0] = 0;
			int recvResult = threadInternal->Recv( buf, 5000 );
			if ( recvResult <= 0 ) {
				break;
			}
			log_message_is( "Received: ", recvResult, buf );
			char* pageToGet = NULL;
			char pageBuf[100];
			for ( int i = 0; i < 100; i++ ) {
				if ( buf[i] == ' ' ) {
					if ( pageToGet == NULL ) {
						pageToGet = &buf[i+1];
					} else {
						buf[i] = 0;
						strcpy( pageBuf, pageToGet );
						buf[i] = ' ';
						break;
					}
				}
			}
			if (( strncmp( &buf[0], "GET", 3 ) == 0 ) && ( pageToGet != NULL )) {
				printf( "getPage: %s\n", pageBuf );
				threadInternal->GetPage( pageBuf );
			} else if ( strncmp( &buf[0], "POST", 4 ) == 0 ) {
				char* endOfHeader = strstr( buf, "\r\n\r\n" );
int counter = 0;
log_message("start of loop" );
				while (( recvResult > 0 ) && ( recvResult2 > 0 ) && ( endOfHeader == NULL )) {
//					Sleep(100);
					recvResult2 = threadInternal->Recv( &buf[ recvResult ], 5000 - recvResult );
					log_message_is( "Second recv result:", recvResult2, &buf[ recvResult ] );
					if ( recvResult2 > 0 ) {
						recvResult += recvResult2;
					}
					endOfHeader = strstr( buf, "\r\n\r\n" );
					counter++;
					if ( counter > 7 ) {
						break;
					}
				}
				log_message("out of loop" );
				if ( !doPost( pageBuf, buf, threadInternal ) ) {
					log_message( "Since I did not fully process form, I must try again" );
					recvResult = threadInternal->Recv( &buf[recvResult], 5000 );
					if ( recvResult <= 0 ) {
						log_message(" this time recvResult is -1" );
						break;
					} else {
						doPost( pageBuf, buf, threadInternal );
					}
				}
			}
			char* keepAlive = strstr( buf, "keep-alive" );
			if ( keepAlive == NULL ) {
				keepAlive = strstr( buf, "Keep-Alive" );
			}
			if ( keepAlive == NULL ) {
				log_message("did not got keep-alive" );
				//threadInternal->Close();
				threadInternal->ShutDown();
			}
	}
	threadInternal->Close();
	printf( "************* THREAD END **************\n" );
}

void MindtelSocketServer::Run() {
	while ( 1 ) {
		printf( "Waiting for a connection\n" );
		if ( AcceptConnection() ) {
			MindtelSocketServer* s = new MindtelSocketServer();
			s->SetSocket( GetSocket() );
			_beginthread(MyThreadProc, 10000, s);

			//Close();
		}
	}
}
  

/**
 *  Very simple page generator.
 */
void MindtelSocketServer::SendPage( char* text ) {
	char header[500];
	char body[500];
	static char* headerFormat = "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\nContent-Length: %d\r\n\r\n";
	static char* bodyFormat = "<HTML><HEAD><TITLE>TEST</TITLE></HEAD><BODY><H3>%s</H3></BODY></HTML>";
	sprintf( body, bodyFormat, text );
	sprintf( header, headerFormat, strlen( body ));
	Send( header, strlen( header ) );
	Send( body, strlen( body ) );
}

static char* headerFormat = "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\nContent-Length: %d\r\n\r\n";
static char* notFoundFormat = "HTTP/1.1 404 Not Found\r\nContent-Type: text/html\r\nContent-Length: %d\r\n\r\n";

void MindtelSocketServer::Redirect( char* location ) {
	char header[500];
	char body[500];
	static char* bodyFormat = "<html><head><meta HTTP-EQUIV=\"REFRESH\" CONTENT=\"0; URL=%s\"></head></html>";
	sprintf( body, bodyFormat, location );
	sprintf( header, headerFormat, strlen( body ));
	Send( header, strlen( header ) );
	Send( body, strlen( body ) );
}

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

/*
 *  MindtelSocketServer::GetPage
 *
 *  Get and serve up a page.  Special handling for "internalValue" including on a page,
 *  and list of neatview visual objects.
 *
 *  Input parmeters:
 *    page -- the name of the page to serve
 */
extern char* genCheckBoxes();
extern char* genRadioBoxes();
void load_config_file( char* configFileName );
char* gen_config_inputs();
char* get_latest_search();
char* get_latest_image();
char* get_hidden_fields();
#define DATASIZE 20000

void MindtelSocketServer::GetPage( char* page ) {
	printf( "\n\nGetting page %s\n", page );
	char filePath[500];
	sprintf( filePath, ".%s", page );
	int fh = -1;
	int fileLength = 0;
	if ( strcmp( filePath, "./" ) == 0 ) {
		strcpy( filePath, "./neatview.html" );
	}
	fh = _open( filePath, _O_RDWR, _S_IREAD | _O_BINARY);
	if ( fh != -1 ) {
		fileLength = _filelength( fh );
		_close( fh );
	} else {
		char* notfound = "<html><body>'%s' not found</body></html>";
		char header[500];
		char body[500];
		sprintf( body, notfound, filePath );
		sprintf( header, notFoundFormat, strlen( body ));
		Send( header, strlen( header ));
		Send( body, strlen( body ));
	}

	FILE* fp = fopen( filePath, "rb" );
	boolean sentHeader = false;
	if (( fileLength > 0 ) && ( fp != NULL )) {
		while ( fileLength > 0 ) {
			char data[DATASIZE];
			memset( &data[0], 0, DATASIZE );
			int count = fread( &data[0], 1, DATASIZE, fp );
			char* internalValueOffset = strstr( &data[0], "<<<%i>>>" );
			if ( internalValueOffset == NULL ) {
				//log_message(" DID NOT FIND SPECIAL INT INCLUDE " );
			} else {
				char buf[50];
				sprintf( buf, "%d", internalValue );
				strncpy( internalValueOffset, "        ", 8 );
				strncpy( internalValueOffset, buf, strlen( buf ));
			}
			char* neatviewEntry = strstr( &data[0], "<<<viz>>>" );
			if ( neatviewEntry != NULL ) {
				int oldLength = strlen( &data[0] );
				log_messagei1( "old count: ", count );
				log_messagei1( "old length: ", oldLength );
				log_messagei1( "old fileLength:",fileLength );
				char* temp = newstrcpy( neatviewEntry );
				char* temp2 = NULL;
				int additional = 9;
				if ( strncmp( temp + 9, "<<<radio>>>", 11 ) == 0 ) {
					temp2 = genRadioBoxes();
					additional += 11;
				} else if ( strncmp( temp + 9, "<<<check>>>", 11 ) == 0 ) {
					temp2 = genCheckBoxes();
					additional += 11;
				} else {
					temp2 = genRadioBoxes();
				}
				strcpy( neatviewEntry, temp2 );
				strcat( neatviewEntry, temp + additional );
				log_messagei1( "new count: ", count );
				log_messagei1( "new length: ", fileLength );
				free( temp );
				free( temp2 );
				fileLength = strlen( &data[0] );
				count = fileLength;
			}

			char* configForm = strstr( &data[0], "<<<config>>>" );
			if ( configForm != NULL ) {
				char* configFileName = configForm + 12;
				char* scanner = configFileName;
				while ( *scanner && (*scanner != ' ')) {
					scanner++;
				}
				*scanner = 0;
				load_config_file( configFileName );
				int additionalLength = strlen( configFileName ) + 1 + 12;
				*scanner = ' ';
				char* temp = newstrcpy( configForm );
				char* temp2 = gen_config_inputs();
				strcpy( configForm, temp2 );
				strcat( configForm, temp + additionalLength );
				free( temp );
				free( temp2 );
				fileLength = strlen( &data[0] );
				count = fileLength;
			}
			char* latestSearch = strstr( &data[0], "<<<search>>>" );
			if ( latestSearch != NULL ) {
				char* temp = newstrcpy( latestSearch );
				char* temp2 = get_latest_search();
				strcpy( latestSearch, temp2 );
				strcat( latestSearch, temp + 12 );
				free( temp );
				free( temp2 );
				fileLength = strlen( &data[0] );
				count = fileLength;
			}
			char* generatedImage = "";
			while ( generatedImage != NULL ) {
				generatedImage = strstr( &data[0], "<<<image>>>" );
				if ( generatedImage != NULL ) {
					log_message("found image tag" );
					char* temp = newstrcpy( generatedImage );
					char* temp2 = get_latest_image();
					strcpy( generatedImage, temp2 );
					strcat( generatedImage, temp + 11 );
					free( temp );
					free( temp2 );
					fileLength = strlen( &data[0] );
					count = fileLength;
				} else {
					log_message("did not find image tag" );
				}
			}
			char* hiddenFields = strstr( &data[0], "<<<hidden>>>" );
			if ( hiddenFields != NULL ) {
				char* temp = newstrcpy( hiddenFields );
				strcpy( hiddenFields, get_hidden_fields() );
				strcat( hiddenFields, temp + 12 );
				free( temp );
				fileLength = strlen( &data[0] );
				count = fileLength;
			}

			if ( count > 0 ) {
				if ( !sentHeader ) {
					char header[500];
					sprintf( header, headerFormat, fileLength);
					Send( header, strlen( header ));
					sentHeader = true;
					log_message1( "Sent header: ", header );
				}
				log_messagei1( "Sending data, count: ", count );

				Send( &data[0], count );
				fileLength -= count;
				log_messagei1( "fileLength is now: ", fileLength );
			} else {
				break;
			}
		}
		fclose( fp );
		//Close();
	} else {
		//printf( "Hey, what is this!\n" );
	}
	//printf( "I got it\n" );
}